/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.DT;

import hypercast.*;
import java.io.*;
import java.util.*;
import java.net.*;

/** Structure used to hold information of each buddy.
 * @author HyperCast Team
 * @author Weisheng Si
 * @version 2.0, Nov 11, 2001 
*/
class BuddyRecord 
{
	/**
	 * The underlay address of the buddy contained in this object.
	 */
	private I_NetworkAddress buddy;

	/**
	 * The connected times of this buddy.
	 */
	private short connected;

	/**
	 * Constructor of BuddyRecord from a network address. 
	 * This address may be a physical address or an underlay 
	 * address.
	 * @param ap 	network address of the buddy.
	 */
	BuddyRecord(I_NetworkAddress ap) 
    {
        buddy = ap;
		connected=0;
    }

	/** 
	 * constructor of BuddyRecord from a string.
	 * the string has to be in syntax ipaddr:port.
	 * @param Adapter 	node adapter.
	 * @param addrstr 	string representing the addres of the buddy.
	 */
	BuddyRecord(I_UnicastAdapter Adapter, String addrstr) 
    {
		/*
		 * Create the underlay address of buddy from a string.
		 */
		buddy = Adapter.createUnderlayAddress(addrstr);

		if (buddy == null)
		{
			throw new HyperCastWarningRuntimeException ("Buddy address is invalid: " + addrstr + "\n");
		}

		connected=0;
    }

	/**
	 * Return the network address of the buddy contained in this object.
	 */
	public I_NetworkAddress getBuddyAddress()
	{
		return buddy;
	}

	/**
	 * Return the connected number of this buddy.
	 */	
	synchronized public short getConnected()
	{
		return connected;
	}
	
	/**
	 * Set the connected number of this buddy to specified value.
	 */	
	synchronized public void setConnected(short conn)
	{
		connected=conn;
	}
	
	/**
	 * Increase the connected number of this buddy by one.
	 */	
	synchronized public void increaseConnected()
	{
		connected++;
	}
	
	/** Creates a string following the "(x,y)@ipaddr/port" syntax. */
	public String toString() 
    {
		return buddy.toString();
    }   
   
   /** 
    * Compares two Buddy records.
	* If they have the same physical address, then we believe they are the same buddy.
    * It is important that the argument is of type <code>Object</code> so that 
    * it can be used in a <code>Hashtable</code>.
    * 
    * @param    obj an object (should be of type <code>BuddyRecord</code>.)
    * @exception IllegalArgumentException if <code>obj</code> is not of type 
	* <code>BuddyRecord</code>.
    */
    public boolean equals(Object obj) {

        if (obj == null)
        {
            return false;
        }
        else if (!(obj instanceof BuddyRecord))
        {
            throw new IllegalArgumentException("Error in equals(): not of type BuddyRecord!");
        }

		BuddyRecord other=(BuddyRecord)obj;
		return buddy.equals(other.getBuddyAddress());
		
	}
}
