/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

/**
 * This defines an interface for address implementations. 
 * It is extended by the interfaces I_PhysicalAddress/I_LogicalAddress.
 * See classes HC_LogicalAddress/IP_Address where these interfaces
 * are actual implemented. 
 *
 * @author HyperCast Team
 * @author Dongwen Wang
 * @author Michael Nahas
 * @version	2.0, Feb. 23, 2001
 */

public interface I_Address {
   /** Checks if an object equals this I_Address .
       * if <code>o</code> is null, false is returned.
       * @throws IllegalArgumentException if Object is not of same type.
       */
    public boolean equals(Object o);

   /** Checks if an I_address object is greater than this one. (Total ordering.)
       * @throws IllegalArgumentException if I_Address is not of same type.
       * @throws IllegalArgumentException if I_Address is null.
       */
    public boolean greaterThan(I_Address a);    

    /** Converts this I_Address to a byte array */
	public byte[] toByteArray();	

    /** Returns the byte array size used to hold this I_Address */
	public int getSize();	
	
    /** Returns the hashcode of this object */
	public int hashCode();
}
