/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

/**
 * This is the interface for an overlay node.  The node is
 * responsible for organizing the overlay and providing information on
 * how to route a message on the overlay. 
 *
 * @author HyperCast Team
 * @author Dongwen Wang
 * @author Jianping Wang
 * @version 3.0, Jun. 08, 2004 
*/
 
public interface I_Node extends I_Stats
{
    /** Default cofiguration file name used by DT_server. */
    public static final String DEFAULT_CONFIG_FILE = "hypercast.xml";

    /** Protocol_Subfield values in Protocol field for DT3.0 protocol message */
    public static final byte PROTOSUB_NOSPEC = 0;
    public static final byte PROTOSUB_SERVER = 1;
    public static final byte PROTOSUB_BUDDYLIST = 2;
    public static final byte PROTOSUB_MULTICAST = 3;


	/** Joins the overlay.  (Includes starting any adapter underneath the node.)
         * @throws IllegalStateException if already joined.
         */
    public void joinOverlay();

	/** Leaves the overlay.  (Includes stopping any adapter underneath the node.)
         * @throws IllegalStateException if already left.
         */
	public void leaveOverlay();
	
	
    /** Returns the addresspair of the next hop for a message routed by this node
         * towards the root.  The next hop would be the node's parent in the spanning tree
         * rooted at <code>root</code>.
         * @throws IllegalStateException if node has not joined group.
         */
	public I_AddressPair[] getParent(I_LogicalAddress root);

    /** Returns the node's children's physical/logical address pairs, with respect to
         * the spanning tree rooted at <code>root</code>.
         * @throws IllegalStateException if node has not joined group.
         */
	public I_AddressPair[] getChildren(I_LogicalAddress root);

    /** Returns the node's neighbors' physical/logical address pairs.
         * @throws IllegalStateException if node has not joined group.
         */
	public I_AddressPair[] getAllNeighbors();
    
    /** Returns this logical and physical addresses of this node. 
         * This can never be null.  (Neither can either the LA or PA part of it.)
         * @throws IllegalStateException if node has not joined group.
         */
	public I_AddressPair getAddressPair();
	
    /** Creates a logical address object from a byte array.
         * The number of bytes read is equal to I_Node.getMyAddressPair().getLogicalAddress().getSize().
         * @throws IllegalArgumentException if byte[] is corrupted 
         * @throws ArrayIndexOutOfBoundsException if byte[] is too small.
         */
	public I_LogicalAddress createLogicalAddress(byte[] laddr, int offset);

	/** Creates a logical address object from a String.
	 */
	public I_LogicalAddress createLogicalAddress(String laStr);

	/** Sets the logical address to specified one.
	 */
	public void setLogicalAddress(I_LogicalAddress la);

	/** Check if previous hop is a valided sender.
	 */

	public boolean previousHopCheck(I_LogicalAddress src, I_LogicalAddress dst, I_LogicalAddress prehop);

	/**
	 * Set notification handler. 
	 */	
	public void setNotificationHandler(NotificationHandler nh);
}



