/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

/**
 * This class exists to allow binding information to be returned to an
 * application. Rather than giving an application a pointer to the
 * internal structures of the naming service object of this class
 * allow an application to access all information about a binding
 * without compromising the encapsulation of data in the
 * implementation of the naming service.
 *
 * @author HyperCast Team
 * @author Greg Mattes
 * @version Spring 2005
 */
public class ImmutableNamingBinding
    implements I_NamingBinding
{
    final boolean local;
    final boolean trusted;
    final boolean authoritative;
    final String  name;
    final String  logicalAddressString;
    final long    timestamp;
    final int     logicalAddressChangeCounter;

    public ImmutableNamingBinding (final boolean local,
                                   final boolean trusted,
                                   final String  name,
                                   final String  logicalAddressString,
                                   final long    timestamp,
                                   final int     logicalAddressChangeCounter,
                                   final boolean authoritative)
    {
         this.local                       = local;
         this.trusted                     = trusted;
         this.name                        = name;
         this.logicalAddressString        = logicalAddressString;
         this.timestamp                   = timestamp;
         this.logicalAddressChangeCounter = logicalAddressChangeCounter;
         this.authoritative               = authoritative;
    }

    public boolean isLocal()                        { return local;                       }
    public boolean isTrusted()                      { return trusted;                     }
    public boolean isAuthoritative()                { return authoritative;               }
    public String  getName()                        { return name;                        }
    public String  getLogicalAddressString()        { return logicalAddressString;        }
    public long    getTimestamp()                   { return timestamp;                   }
    public int     getLogicalAddressChangeCounter() { return logicalAddressChangeCounter; }

    public String toString() {
        final long now = System.currentTimeMillis();
        return
            "<"                                                                     +
            "name"            + " -> " + name                        + ", "         +
            "logical address" + " -> " + logicalAddressString        + ", "         +
            "LA changes"      + " -> " + logicalAddressChangeCounter + ", "         +
            "timestamp"       + " -> " + ((now - timestamp)/1000)    + " sec old, " +
            "trusted"         + " -> " + trusted                     + ", "         +
            "authoritative"   + " -> " + authoritative               +
            ">";
    }
}
