/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

import hypercast.events.NOTIFICATION_EVENT;
import hypercast.util.XmlUtil;

/**
 * This class is a finite state machine which handles Incast.
 *
 * @author HyperCast Team
 * @author Guimin Zhang
 * @version 2.0, May. 23, 2001 */


public class MessageStoreFSM_Incast extends I_MessageStoreFSM
{
		/** OL_Socket object*/
	private OL_Socket socket;
		/** MessageStore*/
	private MessageStore MStore;
		/** logical address*/
	private I_LogicalAddress mylogicaladdress;
		/** root logical address*/
	private I_LogicalAddress root=null;
	/** the message id */
	private byte[] messageid = null;	

		/**ADF_Payload for InCast*/
	public static final byte PAYLOAD						= -1;	

	
	    /** Payload of the message*/
    private byte[] Data = null;
        /** Payload Length*/
    private int datalength = 0;
	
		/** The state information of the message */
	private byte MyState = Init;
		/** the length of Children*/
	private int length;
	
	
		/** The state of the message,Init */
	public static final byte Init = 0;
		/** The state of the message,Wait for ACK*/
	public static final byte Wait = 1;
		/** The state of the message,Done */
	public static final byte Done = 2;

	/** the wait timeout id */
	private static final int WAIT_TIMER_INDEX = 0;
	/**the delete timeout id */
	private static final int DELETE_TIMER_INDEX = 1;
	/**the final delete timeout id */
	private static final int FINALDELETE_TIMER_INDEX = 2;
	/**the routing change timeout id */
	private static final int ROUTINGCHANGE_TIMER_INDEX = 3;
	
	/** The timeout const,Wait(in millisecond) */
	private static long timeout_Wait = -1;
	/** The timeout const,MaxRequest(in millisecond) */
	//private long max_Request;
	/** The timeout const,delete(in millisecond) */
	private static long delete;
	/** The timeout const,final_delete(in millisecond) */
	private static long final_Delete;

    private static FiniteStateMachineMetaOperations metaOps;
    protected static FiniteStateMachineMetaOperations registerMetaOperations ()
    {
        if (metaOps == null)
        {
            metaOps = new IncastMetaOperations();
        }
        return metaOps;
    }

	/**
	 * Constructs a MessageStoreFSM_Incast object with OL_Socket and MessageStore. 
	 */
	public MessageStoreFSM_Incast(OL_Socket socket,MessageStore MStore,HyperCastConfig config)
	{
		/*
		 * Initialization
		 */
		this.socket = socket;
		this.MStore = MStore;
		mylogicaladdress = socket.getLogicalAddress();

        
        /*
         * read in the configuration constant
         */        
		if (timeout_Wait < 0)
		{
			timeout_Wait = config.getPositiveLongAttribute
					(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/Incast/TimeoutWait"));
			delete       = config.getPositiveLongAttribute
					(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/Incast/Delete"));
			final_Delete = config.getPositiveLongAttribute
					(XmlUtil.createXPath("/Public/NetworkService/EnhancedDelivery/Incast/FinalDelete"));
		}


		/*
		 * In case that the finite state machine stuck at the Init state,
		 * it will delete itself when the final_Delete timeout occurs.
		 */
		MStore.setTimer(this,FINALDELETE_TIMER_INDEX,final_Delete);
	}
	
	/** 
	 * This method is used to adapt the dynamically changing tree
	 * structure of the overlay topology. It will take different actions
	 * depending on the current state of the finite state machine.
	 */	
	public void changingNeighbor()
	{
		//no body
	}  
	

	/**
	 * @return the finite state machine id
	 */
	public short getFSMID()
	{
		return MessageStore.FSM_INCAST;
	}
	

	/**Set the new ADF_Control message entry
	 * the information buffered includes the message id, the root of the tree
	 * @param	msg			OL_Message
	 */
	private void setACMsg(OL_Message msg)
	{
		//no control message
	}
	/**Set the new ADF_Data message entry
	 * the information buffered includes the message id, the root of the tree
	 * and the payload data 
	 * @param	msg			OL_Message
	 */
	private synchronized void setADMsg(OL_Message msg)
	{
		//message id
		messageid = msg.getMessageIdentifier();
		//root
		root = msg.getDestinationAddress();

		/*
		 * concatinate the data
		 */
		byte [] temp = new byte[datalength+msg.getPayloadLength()];
		int len = datalength + msg.getPayloadLength();
		if(Data !=null)
		{
			System.arraycopy(Data,0,temp,0,Data.length);
		}
		if(msg.getPayload()!=null)
		{
			System.arraycopy(msg.getPayload(),0,temp,datalength,msg.getPayloadLength());	
		}
		datalength = len;
		Data = new byte[datalength];
		System.arraycopy(temp,0,Data,0,datalength);	
	}


	/**
	 * This is the method used to forward data to parent node.
	 * It can only be called when the root, message id are initiated.
	 */
	private synchronized void forwardData()
	{
		/*
		 * construct the payload message
		 */
		//OL_Message msg = new OL_Message((short)1, mylogicaladdress);
		OL_Message msg = (OL_Message) socket.createMessage(Data);;
		msg.setHopLimit((short) 1);
		msg.setDeliveryMode(OL_Message.DELIVERY_MODE_UNICAST);
		msg.setDestinationAddress(socket.getParent(root));
		FSM_Extension ee = new FSM_Extension(MessageStore.FSM_INCAST,PAYLOAD,messageid);
		msg.addExtension(ee);
		//msg.addExtension(Extension.createExtension(Extension.RAW_DATA,Data));
		
		/*
		 * if current node is not root,
		 * forward to the parent node.
		 */
		if(!mylogicaladdress.equals(root))
		{
			socket.forwardToParent(msg);
					
		}
		/*
		 * arrived the root, forward to the application
		 */	
		else 
		{
			socket.appmsgArrived(msg,socket.callback); 
		}
	}
		

	/**process new ADF_Data message
	 * @param	msg			OL_Message
	 */
	public void newADMsg(OL_Message msg)
	{
		/*
		 * Buffer the essential information such as message id, root of the tree
		 * and the payload 
		 */
		setADMsg(msg);
		MyState = Wait;
		MStore.setTimer(this,WAIT_TIMER_INDEX,timeout_Wait);		
	}
	

	/**process new ADF_Control message
	 * @param	msg			OL_Message
	 */
	public void newACMsg(OL_Message msg)
	{
		// no body
	 }
	
	
	
	/**Update new ADF_Data message
	 * @param	msg			OL_Message
	 */
	public void updateADMsg(OL_Message msg)
	{
		/*
		 * store the payload
		 */
		setADMsg(msg);
		/*
		 * extract the format message header information
		 */
		FSM_Extension e = (FSM_Extension) msg.getFirstExtensionByType(Extension.FSM);

		switch (MyState) 
		{
			case Init:
			case Wait:
			{
				MStore.setTimer(this,WAIT_TIMER_INDEX,timeout_Wait);
			}			
				break;
			case Done:
			break;
		}
	}
	
	/**process new ADF_Control message
	 * @param	msg			OL_Message
	 */
	public void updateACMsg(OL_Message msg)
	{
		// no body
	}

	
	
	/** process the timer expired events
	 */
	public void timerExpired(int timerIndex)
	{	 

		switch(MyState)
		{
			case Init:
			{
				if(timerIndex==FINALDELETE_TIMER_INDEX)
				{
					MStore.remove(messageid);
				}
			}
				break;
			case Wait:
			{
				if((timerIndex==WAIT_TIMER_INDEX))
				{
					forwardData();
					MyState = Done;
				}
				
			}
				break;
			
			case Done:
			{
				if(timerIndex==DELETE_TIMER_INDEX)
				{
				   MyState = Init;
				   /*
					* delete itself
					*/
				   MStore.remove(messageid);
				}
			}
			break;
			
		}
		
	}
}

class IncastMetaOperations implements FiniteStateMachineMetaOperations
{
    public boolean messageStoreWillForwardMessage (OL_Message message) { return false; }

    public boolean isValidMessage (OL_Message message)
    {
        return message.getDeliveryMode() == OL_Message.DELIVERY_MODE_UNICAST;
    }
	public boolean processIntermediateUnicastMessage(OL_Message message){return true;}
    public NOTIFICATION_EVENT notifyOnSend (OL_Message message) { return null; }
}
