/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.NONE;

import java.util.ArrayList;
import java.util.StringTokenizer;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.apache.xpath.XPath;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.I_AddressPair;
import hypercast.I_LogicalAddress;
import hypercast.I_Node;
import hypercast.I_PhysicalAddress;
import hypercast.I_UnicastAdapter;
import hypercast.NotificationHandler;

import hypercast.I_Stats;
import hypercast.StatsElement;
import hypercast.StatsProcessor;
import hypercast.HyperCastConfigException;
import hypercast.SimpleStats;
import hypercast.util.XmlUtil;

/**
 * @author HyperCast Team
 * @author Guangyu Dong
 * @version 2005 (version 3.0)
 */
public class NONE_Node
    implements I_Node
{
    private HyperCastConfig config;
    private I_UnicastAdapter adapter;
    private NONE_AddressPair addressPair;

    /** pre-configured neighbors: TODO: why can't these be dynamic? */
    private NONE_AddressPair [] neighbors;

    private String PROPERTY_PROTO_PREFIX = "/Public/Node/NONE/";

	/**
	 * A StatsProcessor instance which handles the statistics in this object.
	 */
    private StatsProcessor statsPro;

	/**
	 * The statistics name for this object (as an I_Stats instance). Its default
	 * name is defined in the socket configuration schema file. It can be assigned
	 * by any other object through method setStattisticsName defined in the 
	 * I_Stats interface.
	 */
	private String statisticsName;


    public NONE_Node (final HyperCastConfig config, final I_UnicastAdapter adapter)
    {
        this.config = config;
        this.adapter = adapter;
        this.addressPair = new NONE_AddressPair (new NONE_LogicalAddress (adapter.createPhysicalAddress()));
		
		String neighborsString = "";
		try
		{
			neighborsString = config.getTextAttribute
					(XmlUtil.createXPath(PROPERTY_PROTO_PREFIX + "Neighbors"));
		}
		catch (HyperCastConfigException e)
		{
			/*
			 * The attribute doesn't exist.
			 */
			neighborsString = "";
		}

        initNeighbors (neighborsString);

        initStatisticsStructure();
    }

    public NONE_Node (final HyperCastConfig config, final I_UnicastAdapter adapter, final String prefix)
    {
        this.config = config;
        this.adapter = adapter;
        this.addressPair = new NONE_AddressPair (new NONE_LogicalAddress (adapter.createPhysicalAddress()));
        PROPERTY_PROTO_PREFIX = prefix;
		
		String neighborsString = "";
		try
		{
			neighborsString = config.getTextAttribute
					(XmlUtil.createXPath(PROPERTY_PROTO_PREFIX + "Neighbors"));
		}
		catch (HyperCastConfigException e)
		{
			/*
			 * The attribute doesn't exist.
			 */
			neighborsString = "";
		}

        initNeighbors (neighborsString);

        initStatisticsStructure();
    }

    private void initNeighbors (final String neighborsString)
    {
        final ArrayList neighborsArray = new ArrayList();
        final String SEPARATOR = ";";

		final StringTokenizer tokenizer = new StringTokenizer (neighborsString.trim(), SEPARATOR);
        while (tokenizer.hasMoreTokens())
        {
            final String neighborEntryString = tokenizer.nextToken();

            System.err.println ("Adding " + neighborEntryString + " into NONE neighborhood table...");

            final I_PhysicalAddress neighborEntryPhysicalAddress = adapter.createPhysicalAddress (neighborEntryString);
            neighborsArray.add (new NONE_AddressPair (new NONE_LogicalAddress (neighborEntryPhysicalAddress)));
        }
        neighbors = (NONE_AddressPair []) neighborsArray.toArray (new NONE_AddressPair [neighborsArray.size()]);
    }

    /**
     * This is a no-op for the NONE protocol since there is no overlay to join.
     *
     * @see hypercast.I_Node#joinOverlay()
     */
    public void joinOverlay()
    {
        config.log.println ("NONE_Node started: " + addressPair);
        config.log.flush();
    }

    /**
     * This is a no-op for the NONE protocol since there is no overlay to join.
     *
     * @see hypercast.I_Node#leaveOverlay()
     */
    public void leaveOverlay()
    {
    }

    /**
     * 
     *
     * @see hypercast.I_Node#getParent(hypercast.I_LogicalAddress)
     */
    public I_AddressPair [] getParent (final I_LogicalAddress root)
    {
        //System.out.println("getParent for "+root);
		
        return new I_AddressPair [] { new NONE_AddressPair ((NONE_LogicalAddress) root) };
		
        //final I_AddressPair [] result = new I_AddressPair [1];
        //result[0] = new NONE_AddressPair ((NONE_LogicalAddress) root);
		
        //System.out.println("parent= "+result[0].getPhysicalAddress());
		
        //return result;
    }

    /**
     * @see hypercast.I_Node#getChildren(hypercast.I_LogicalAddress)
     */
    public I_AddressPair [] getChildren (final I_LogicalAddress root)
    {
        return root.equals (addressPair.getLogicalAddress()) ? neighbors : null;
    }

    /**
     * @see hypercast.I_Node#getAllNeighbors()
     */
    public I_AddressPair [] getAllNeighbors()
    {
        return neighbors;
    }

    /**
     * @see hypercast.I_Node#getAddressPair()
     */
    public I_AddressPair getAddressPair()
    {
        return addressPair;
    }

    /**
     * @see hypercast.I_Node#createLogicalAddress(byte[], int)
     */
    public I_LogicalAddress createLogicalAddress (final byte [] logicalAddressBytes, final int offset)
    {
        return new NONE_LogicalAddress (adapter.createPhysicalAddress (logicalAddressBytes, offset));
    }

    /**
     * Everybody is considered a neighbor in
     * the NONE protocol. All previous hops are considered to be valid
     * unless the previous hop is this node.
     *
     * @see hypercast.I_Node#previousHopCheck(hypercast.I_LogicalAddress, hypercast.I_LogicalAddress, hypercast.I_LogicalAddress)
     */
    public boolean previousHopCheck (final I_LogicalAddress source, final I_LogicalAddress destination, final I_LogicalAddress previousHop)
    {			
        I_LogicalAddress myLogicalAddress = addressPair.getLogicalAddress();
        return ! source.equals (myLogicalAddress);
    }

    /**
     * @see hypercast.I_Node#setNotificationHandler(NotificationHandler)
     */
    public void setNotificationHandler (final NotificationHandler notificationHandler)
    {
        // TODO: could NONE generate notifications?
        //throw new UnsupportedOperationException();
    }

    /**
     * @see hypercast.I_Node#createLogicalAddress(java.lang.String)
     */
    public I_LogicalAddress createLogicalAddress (final String logicalAddressString)
    {
        return new NONE_LogicalAddress (adapter.createPhysicalAddress (logicalAddressString));
    }

    /**
     * Sets the logical address of this node to the specified one.
     */
    public void setLogicalAddress (final I_LogicalAddress logicalAddress)
    {
        if (! (logicalAddress instanceof NONE_LogicalAddress))
            throw new IllegalArgumentException ("NONE_Node.setLogicalAddress: the logical address type of the parameter is incorrect.");

        addressPair = new NONE_AddressPair ((NONE_LogicalAddress) logicalAddress);
    }

	private void initStatisticsStructure()
	{
		/*
		 * Get the statistics name defined in the configuration file for this I_Stats instance.
		 */
		statisticsName = config.getTextAttribute (XmlUtil.createXPath(PROPERTY_PROTO_PREFIX+"StatName"));

		/*
		 * Initial StatsProcessor instance.
		 */
		statsPro = new StatsProcessor(this/* I_Stats object */, true/* readable flag */, true/* writable flag */);
		
		statsPro.addStatsElement("NodeAdapter"/* name */, adapter/* I_Stats object */, 1/* minoccurs */, 1/* maxoccurs */);
		
		statsPro.addStatsElement("Neighbors", new CurrentNeighbors() ,1, 1);

		
		statsPro.addStatsElement("LogicalAddress", new LogicalAddressOperator(), 1, 1);

		statsPro.addStatsElement("PhysicalAddress", new PhysicalAddressOperator(), 1, 1);

		statsPro.addStatsElement("NumOfNeighbors", new NumOfCurrentNeighbors(), 1, 1);

	}



	/** @see I_Stats#getStatsName() */
	public String getStatsName ()
	{
		return statisticsName;
	}

	/** @see I_Stats#setStatsName(String) */
	public void setStatsName (String name)
	{
		statisticsName = name;
	}

    /**
     * Gets the information of a scalar statistics from an object which 
     * implements this interface.
     * @throws HyperCastStatsException if the statistics with given name is not
     * supported by the object.
     */
    public String getStatsValue (final String name)
        throws HyperCastStatsException
    {
        if (name.equals ("Neighbors"))
        {
            final StringBuffer buffer = new StringBuffer();

            for (int i = 0; i < neighbors.length; i++)
            {
                buffer.append (neighbors[i].getLogicalAddress().toString());
                if (i < (neighbors.length - 1))
                {
                    buffer.append (";");
                }
            }
            return buffer.toString();
        }
        else if (name.equals ("LogicalAddress"))
        {
            return addressPair.getLogicalAddress().toString();
        }
        else if (name.equals ("PhysicalAddress"))
        {
            return addressPair.getPhysicalAddress().toString();
        }
        else if (name.equals ("NumOfNeighbors"))
        {
            return Integer.toString (neighbors.length);
        }
        else
        {
            throw new HyperCastStatsException(name, HyperCastStatsException.StatisticNotFound);
        }
    }

	/**
	 * Sets the information of s writable scalar statistics to an object which 
	 * implements this interface.
	 * @throws HyperCastStatsException if the statistics with given name is not
	 * supported by the object or read-only, or if value does not conform with 
	 * expected format.
	 */
	public String setStatsValue (final String name, final String newValue)
            throws HyperCastStatsException
	{
            if (name.equals ("Neighbors"))
            {
                try
                {
                    final String result = newValue.trim().equals ("none") ? "" : newValue;
                    this.initNeighbors (result);
				
                    return result;
                }
                catch (final Exception e)
                {
                    throw new HyperCastStatsException (name + "Unable to initiate Neighbors with: " + newValue + ": " + e, HyperCastStatsException.InappropriateValue);
                }

            }
            else
            {
                throw new HyperCastStatsException(name, HyperCastStatsException.StatisticNotFound);
            }
	}

	/** @see I_Stats#getStats */
	public Element[] getStats (final Document doc, final XPath xpath)
            throws HyperCastStatsException
	{
		return statsPro.getStatsResult (doc, xpath);
	}

	/**@see I_Stats#setStats */
	public Element[] setStats (final Document doc, final XPath xpath, final Element newValue)
            throws HyperCastStatsException
	{
		return statsPro.setStatsResult(doc, xpath, newValue);
	}

	/** @see I_Stats#getReadSchema */
	public Element[] getReadSchema (final Document doc, final XPath xpath)
            throws HyperCastStatsException
	{
		return statsPro.getReadSchemaResult(doc, xpath);
	}

	/** @see I_Stats#getWriteSchema */
	public Element[] getWriteSchema (final Document doc, final XPath xpath)
            throws HyperCastStatsException
	{
		return statsPro.getWriteSchemaResult(doc, xpath);
	}
	

	/**
	 * This class manages simple statistic "Neighbors".
	 */
	class CurrentNeighbors extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			final StringBuffer buffer = new StringBuffer();

			for (int i = 0; i < neighbors.length; i++)
			{
				buffer.append (neighbors[i].getLogicalAddress().toString());
				if (i < (neighbors.length - 1))
				{
					buffer.append (";");
				}
			}
			return buffer.toString();
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			try
			{
				final String result = newValue.trim().equals ("none") ? "" : newValue;
				initNeighbors (result);
				
				return result;
			}
			catch (final Exception e)
			{
				throw new HyperCastStatsException (statisticsName + "Unable to initiate Neighbors with: " + newValue + ": " + e, HyperCastStatsException.InappropriateValue);
			}
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:String"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "LogicalAddress".
	 */
	class LogicalAddressOperator extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return addressPair.getLogicalAddress().toString();
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:String"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}


	/**
	 * This class manages simple statistic "PhysicalAddress".
	 */
	class PhysicalAddressOperator extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return addressPair.getPhysicalAddress().toString();
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:String"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}


	/**
	 * This class manages simple statistic "NumOfNeighbors".
	 */
	class NumOfCurrentNeighbors extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return Integer.toString (neighbors.length);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Integer"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

}



