/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.SPT;

import java.util.Enumeration;
import java.util.Hashtable;

import hypercast.HyperCastConfig;
import hypercast.I_UnicastAdapter;
import hypercast.HyperCastConfigException;
import hypercast.util.XmlUtil;

/**
 * This class deals with the topology algorithm of SPT protocol
 * 
 * @author HyperCast Team
 * @author Guangyu Dong
 * @version 2005 (version 3.0)
 */
class SPT_TopologyPolicy {
	
	/** the config object */
	HyperCastConfig config;
	/** for getting time */
	I_UnicastAdapter adapter;
	/** my logical address */
	SPT_LogicalAddress myLA;
	
	
	
	final static String PROP_NAME_TOPO_POLICY="TopologyPolicy";
	final static String PROP_NAME_JUMP_THRESHOLD="JumpThreshold";
	
	final static String LINK_QUALITY_PREFIX="LinkQuality";
	final static String PROP_NAME_PING_BUF_SIZE="PingBufSize";
	final static String PROP_NAME_RELIABLE_THRESHOLD="ReliableThreshold";
	
	/** the topology policy */
	String Topo_Policy;
	/** 
	 * The node will switch parent if a neighbor has a better path metric by this
	 * threshold value than current parent.
	 */ 
	int Jump_Threshold;
	
	/** properties for LinkQuality */
	int LQ_PingBufSize;
	float LQ_ReliableThreshold;
	long LQ_PingInterval;
	
	/** a node with this address will always become parent */
	SPT_LogicalAddress NextParent;
	
	public SPT_TopologyPolicy(HyperCastConfig c, String pre, SPT_LogicalAddress la, I_UnicastAdapter adpt){
		config=c;
		myLA=la;
		adapter=adpt;
		
		//String prefix=pre+".";
		String prefix=pre;
		
		//Topo_Policy=config.getStringProperty(prefix+PROP_NAME_TOPO_POLICY, PathQuality_PathMetric.Policy_Name);
		try
		{
			Topo_Policy=config.getTextAttribute
					(XmlUtil.createXPath(prefix+PROP_NAME_TOPO_POLICY));
		}
		catch (HyperCastConfigException e)
		{
			Topo_Policy = PathQuality_PathMetric.Policy_Name;
		}

		if(!Topo_Policy.equals(MinimumCost_PathMetric.Policy_Name) && !Topo_Policy.equals(PathQuality_PathMetric.Policy_Name) &&!Topo_Policy.equals(LinkQuality_PathMetric.Policy_Name)){
			throw new IllegalArgumentException("Illegal Topology Policy: "+Topo_Policy);
		}else{
			config.log.println("TopologyPolicy="+Topo_Policy);
		}
		
		//Jump_Threshold=config.getIntProperty(prefix+PROP_NAME_JUMP_THRESHOLD);
		Jump_Threshold=config.getIntAttribute
				(XmlUtil.createXPath(prefix+PROP_NAME_JUMP_THRESHOLD));
		
		//prefix=pre+LINK_QUALITY_PREFIX+".";
		prefix=pre+LINK_QUALITY_PREFIX+"/";
		
		//LQ_PingBufSize=config.getIntProperty(prefix+PROP_NAME_PING_BUF_SIZE);
		LQ_PingBufSize=config.getIntAttribute
				(XmlUtil.createXPath(prefix+PROP_NAME_PING_BUF_SIZE));

		//LQ_ReliableThreshold=Float.parseFloat(config.getStringProperty(prefix+PROP_NAME_RELIABLE_THRESHOLD, "0.1"));
		LQ_ReliableThreshold=Float.parseFloat(
				config.getTextAttribute(XmlUtil.createXPath(prefix+PROP_NAME_RELIABLE_THRESHOLD)));

		//LQ_PingInterval=config.getPositiveLongProperty(pre+"."+SPT_Node.BEACON_TIME_PROPERTY_NAME, SPT_Node.BEACON_TIME_DEFAULT);
		LQ_PingInterval=config.getPositiveLongAttribute
				(XmlUtil.createXPath(pre+SPT_Node.BEACON_TIME_PROPERTY_NAME));
		
	}
	
	/**
	 * create a path metric measurement object
	 */
	public I_PathMetric createPathMetric(SPT_Message beacon, LinkQuality link){
		if(Topo_Policy.equals(MinimumCost_PathMetric.Policy_Name)){
			return new MinimumCost_PathMetric(beacon);
		}
		else if(Topo_Policy.equals(LinkQuality_PathMetric.Policy_Name)){
			return new LinkQuality_PathMetric(beacon, link);
		}
		else if(Topo_Policy.equals(PathQuality_PathMetric.Policy_Name)){
			
			return new PathQuality_PathMetric(beacon, link);
		}
		else 
			return null;
			
	}
	/**
	 * create a link quality measurement object
	 */
	public LinkQuality createLinkQuality(){
		return new LinkQuality(LQ_PingBufSize, LQ_PingInterval, LQ_ReliableThreshold, myLA, adapter);
	}
	/** 
	 * Compare if this neighbor is better than the current parent
	 * @param ne			the neighbor
	 * @param currentParent the current parent
	 * @return	true/false
	 */
	public boolean isBetterThanCurrentParent(SPT_NeighborhoodEntry ne, SPT_NeighborhoodEntry currentParent){
		if (ne.getLogicalAddress().equals(NextParent)) {
			//NextParent=null;
			return true;
		} else if (NextParent != null) {
			return false;
		}

		if (ne == currentParent)
			return false;

		if (ne == null)
			return false;
		
		if (ne.getPrevMsgTimeStamp()>=ne.getTimeStamp())
			return false;
		
		if (!ne.getLinkQuality().isReliable()){
			return false;
		}
		
		if (currentParent == null)
			return ne.getRootLogicalAddress().isBetterThan(myLA);
		
		if(currentParent.fixed)return false;
		
		if (ne.getParentLogicalAddress()!=null && ne.getParentLogicalAddress().equals(myLA))
			return false;
		
		
		if (ne.getRootLogicalAddress().isBetterThan(currentParent.getRootLogicalAddress())) {
			return true;
		} else if (ne.getRootLogicalAddress().equals(currentParent.getRootLogicalAddress())) {
			//			changed 04-19-04
			if (ne.getPathMetric() >= currentParent.getPathMetric() + this.Jump_Threshold) {
				if(ne.getCost() <= currentParent.getCost() + 2)
					return true;
			}
			
		}
		return false;
	}
	
	/**
	 * to compare two neighbors
	 */
	public boolean isBetterThan(SPT_NeighborhoodEntry ne1, SPT_NeighborhoodEntry ne2){
		
		if (ne1 == ne2)
			return false;

		if (ne1 == null)
			return false;

		if (ne2 == null)
			return true;
		
		if (ne1.getRootLogicalAddress().isBetterThan(ne2.getRootLogicalAddress())) {
			return true;
		} else if (ne1.getRootLogicalAddress().equals(ne2.getRootLogicalAddress())) {
			if (ne1.getPathMetric() > ne2.getPathMetric()) {
				return true;
			}
		}
		return false;
	}
	
	
}


class MinimumCost_PathMetric implements I_PathMetric{
	
	static final String Policy_Name="Cost"; 
	
	SPT_Message lastBeacon;
	
	/**
	 * Constructor
	 * @param beacon	the last received beacon message
	 */
	public MinimumCost_PathMetric(SPT_Message beacon){
		lastBeacon=beacon;
	}
	
	/* (non-Javadoc)
	 * @see hypercast.SPT.I_PathMetric#getPathMetric()
	 */
	public int getPathMetric() {
		if(lastBeacon==null){
			//the starting metric value
			return FULL_PATHMETRIC_VALUE;
		}else{
			return FULL_PATHMETRIC_VALUE-lastBeacon.getCost();
		}
	}

	/* (non-Javadoc)
	 * @see hypercast.SPT.I_PathMetric#beaconReceived(hypercast.SPT.SPT_Message)
	 */
	public void beaconReceived(SPT_Message beacon) {
		lastBeacon=beacon;
	}
	
	
}


class LinkQuality_PathMetric implements I_PathMetric{

	static final String Policy_Name="LinkQuality"; 
	
	SPT_Message lastBeacon;
	
	LinkQuality link;
	
	/**
	 * Constructor
	 * @param beacon	the last received beacon message
	 */
	public LinkQuality_PathMetric(SPT_Message beacon, LinkQuality lnk){
		lastBeacon=beacon;
		link=lnk;
	}
	
	/* (non-Javadoc)
	 * @see hypercast.SPT.I_PathMetric#getPathMetric()
	 */
	public int getPathMetric() {
		if(lastBeacon==null){
			//the starting metric value
			return FULL_PATHMETRIC_VALUE;
		}else{
			
			int linkQuality=(int)(link.getBiLinkQuality()*FULL_PATHMETRIC_VALUE);
			//int pathQuality=lastBeacon.getPathMetric()*linkQuality/FULL_PATHMETRIC_VALUE;
			return linkQuality;
		}
	}

	/* (non-Javadoc)
	 * @see hypercast.SPT.I_PathMetric#beaconReceived(hypercast.SPT.SPT_Message)
	 */
	public void beaconReceived(SPT_Message beacon) {
		lastBeacon=beacon;
	}
	
	
}

class PathQuality_PathMetric implements I_PathMetric{

	static final String Policy_Name="PathQuality"; 
	
	SPT_Message lastBeacon;
	
	LinkQuality link;
	
	/**
	 * Constructor
	 * @param beacon	the last received beacon message
	 */
	public PathQuality_PathMetric(SPT_Message beacon, LinkQuality lnk){
		lastBeacon=beacon;
		link=lnk;
	}
	
	/* (non-Javadoc)
	 * @see hypercast.SPT.I_PathMetric#getPathMetric()
	 */
	public int getPathMetric() {
		if(lastBeacon==null){
			//the starting metric value
			return FULL_PATHMETRIC_VALUE;
		}else{
			//System.out.println("Link to "+lastBeacon.getSrcLA()+" LQ="+link.getBiLinkQuality()+" PQ="+lastBeacon.getPathMetric());
			
			int linkQuality=(int)(link.getBiLinkQuality()*FULL_PATHMETRIC_VALUE);
			int pathQuality=lastBeacon.getPathMetric()*linkQuality/FULL_PATHMETRIC_VALUE;
			return pathQuality;
		}
	}

	/* (non-Javadoc)
	 * @see hypercast.SPT.I_PathMetric#beaconReceived(hypercast.SPT.SPT_Message)
	 */
	public void beaconReceived(SPT_Message beacon) {
		lastBeacon=beacon;
	}
	
	
}

