/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;

import java.io.*;
import java.util.*;
import java.net.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

	/**
	 * This class defines the extra properties of a statistic. These 
	 * extra properties, such as minimal occurance times of the statistic,
	 * cannot be determined by the I_Stats object itself, instead, they 
	 * are determined by the upper layer statistic.
	 *
	 * @author HyperCast Team
	 */
	public class StatsElement
	{
		/**
		 * Minimal occurence times of this element in statistics tree. There is no
		 * definition on minOccurs in the schema element if setting minOccurs to -1.
		 */
		int minOccurs = 1;
	
		/**
		 * Maximal occurence times of this element in statistics tree.  There is no
		 * definition on maxOccurs in the schema element if setting maxOccurs to -1.
		 */
		int maxOccurs = 1;
	
		/**
		 * The I_Stats object which implements this element. It must
		 * not be null. 	
		 */
		I_Stats statsObj;
		
		/**
		 * Constructor.
		 */
		public StatsElement(String Name, I_Stats statisticsObj, int minoccur, int maxoccur)
		{
			statsObj = statisticsObj;
			
			minOccurs = minoccur;
			maxOccurs = maxoccur;
		}

		/** Return the name of this statistics */
		public String getName() {
			if (statsObj != null)
			{
				return statsObj.getStatsName();
			}
			else
			{
				return null;
			}
		}
		
		/** Set the name of this statistics */
		public void setName(String nm) {
			if (statsObj != null)
			{
				statsObj.setStatsName(nm);
			}
		}

		/** Return the I_Stats object stored in this statistics */
		public I_Stats getStatsObj() {
			return statsObj;
		}


		/** Reset the attribute of the given schema element. */
		public void resetSchemaAttribute(Element schemaElement)
		{
			if (maxOccurs == -1)
			{
				schemaElement.setAttribute("maxOccurs", "unbounded");
			}
			else if ((maxOccurs >= 0) && (maxOccurs != 1))
			{
				schemaElement.setAttribute("maxOccurs", ""+maxOccurs);
			}

			if (minOccurs  == -1)
			{
				schemaElement.setAttribute("minOccurs", "unbounded");
			}
			else if ((minOccurs >= 0) && (minOccurs != 1))
			{
				schemaElement.setAttribute("minOccurs", ""+minOccurs);
			}
		}
	}

