/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast;


import java.net.*;
import java.io.*;
import java.util.*;

/**
 * This class is timer ID 
 
 * @author HyperCast Team
 * @author Guimin Zhang
 * @version 2.0, May. 23, 2001
 */
public class Timer_ID{
		/**the finite state machine . */
private I_MessageStoreFSM fsm;
    /** Identifier of the timeout. */
private int id;
    private Object obj = null;
    /**
	 * Constructs a Time_ID object with time id and time.
	 * @param   FSM      finite state machine
	 * @param	ID       time id
	 */
	
	public Timer_ID(I_MessageStoreFSM FSM, int ID) {
		id = ID;
		fsm = FSM;
	}

    /**
     * Construct a timer object with another object attached to it
     */
	public Timer_ID(I_MessageStoreFSM FSM, Object obj) {
            this.id = -1;
            this.obj = obj;
		fsm = FSM;
	}


	/**
	 * Constructs a new Timer_ID object with a Timer_ID object
	 * @param	te	a Timer_ID object
	 */
	public Timer_ID(Timer_ID te) {
		id = te.getID();
                this.obj = te.obj;
		fsm = te.getFSM();
	}
/**
	 * Returns the timeout id 
	 * @return	id
	 */
	public int getID() {
		return id;
	}

    /**
     * Returns the object associated with the timer id object, or null if there is no object associated.
     */
	public Object getObject() {
		return obj;
	}
	
	/**
	 * Returns the finite state machine
	 * @return	fsm
	 */
	public I_MessageStoreFSM getFSM() {
		return fsm;
	}
	
	/**
	 * compare two Timer_ID
	 * @return	true/false
	 */
	public boolean equals(Object o)  {
        if (o == null)  
            return false;
        
        // could be object of type QueryTimer for example
        if (!(o instanceof Timer_ID))
            return false;
        
        Timer_ID td = (Timer_ID) o;
        if (obj != null && td.obj != null)
            return fsm == td.fsm && obj.equals (td.obj);
        else
            return fsm == td.fsm && id == td.id;
    }
}
