/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

package hypercast.adapters;

import hypercast.*;

import java.io.*;
import java.util.*;
import java.net.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.apache.xpath.XPath;

import hypercast.util.XmlUtil;


/** 
 * This class extends the UDP_UnicastAdapter_INETv4AndOnePort.
 * It handles sending messages to a multicast address.
 * It starts a MulticastReceive Thread to receive multicast 
 * messages from a multicast socket.
 * 
 * @author HyperCast Team
 * @author Jianping Wang
 * @version 3.0, Feb. 02, 2004
*/

public class UDP_MulticastAdapter extends UDP_UnicastAdapter implements I_MulticastAdapter {
		
		/** Total number of packets received from the multicast socket */
    long MReceivedPackets;
		/** Total number of packets sent to the multicast socket */
    long MSentPackets;
		/** Total number of bytes received from the multicast socket */
    long MReceivedBytes;
		/** Total number of bytes sent to the multicast socket */
    long MSentBytes;
	
		/** Default Multicast Address is in protperty file */
	INETv4OnePort multicastAddress; 	

		/** Multicast Socket */
	private MulticastSocket MSock = null;
		/** Multicast receiver object */
	private MulticastReceiver MRecv = null;
	
	
	/** 
	 * Constructs a UDP_MulticastAdapter object with a HyperCastConfig
	 * object and an I_UnicastAdapter object.<p>
	 * 
	 * @param	config		a HyperCastConfig object.
	 * @param	scktAdpt	object representing the Socket Adapter. It is set to null when 
	 * 						creating the Socket Adapter.
	 * @param	AddrString	string containing physical address information.
	 * @param	prefix		property prefix.
	 * 						sockets.
	 */
	public UDP_MulticastAdapter(HyperCastConfig config, I_UnicastAdapter scktAdpt, String AddrString, String prefix){
		super(config, scktAdpt, AddrString, prefix);

		MulticastAdapter_Inilization(config, AddrString, prefix);
	}


	/** 
	 * Constructs a UDP_MulticastAdapter object with a HyperCastConfig 
	 * and DataGramSocket object.
	 * @param	config		a HyperCastConfig object.
	 * @param	usock		a DatagramSocket object.
	 * @param	prefix		property prefix.
	 */
	public UDP_MulticastAdapter(HyperCastConfig config, DatagramSocket usock, String prefix){
		super(config, usock, prefix);

		MulticastAdapter_Inilization(config, null, prefix);
	}

	/** 
	 * Finish the initialization for this UDP_MulticastAdapter object.
	 * 
	 * @param	config		a HyperCastConfig object.
	 * @param	AddrStr		string containing address information for underlying
	 * 						unicast socket.
	 * @param	prefix		property prefix.
	 */
	public void MulticastAdapter_Inilization(HyperCastConfig config, String AddrStr, String prefix){

		MReceivedPackets = 0;
		MReceivedBytes = 0;
		MSentPackets = 0;
		MSentBytes = 0;
		
		property = config;

		String multicastAddressString;
		
		String addrType = config.getTextAttribute
				(XmlUtil.createXPath(PROPERTY_NAME_PREFIX + "UDPMulticastAddress"));
		
		multicastAddressString = property.getTextAttribute
				(XmlUtil.createXPath(PROPERTY_NAME_PREFIX +"UDPMulticastAddress/" +addrType));		

		//create the multicast address from a string
		CreateMulticastAddress(multicastAddressString);


		// Create the Multicast Socket
		try {
			MSock = new MulticastSocket(multicastAddress.getPort()); 
		} catch (IOException e) {
                    throw new HyperCastFatalRuntimeException ("UDP_MulticastAdapter: Could not create Multicast Socket");
		}
		try {
			MSock.joinGroup(multicastAddress.getInetAddress());
		} catch (IOException e) {
                    throw new HyperCastFatalRuntimeException ("UDP_MulticastAdapter: Could not join Multicast Group");
		}
		
		InitStatisticsStructure();
	}		
	
	/** 
	 *  Start the Multicast receiver thread <br>
	 */
		
	public void Start() {
		super.Start();

        /*
         * Object for receiving from multicast socket.
         */
		MRecv = new MulticastReceiver(InputChannel, MSock, this);
		Thread MThread = new Thread(MRecv);
		MThread.start();
	}

	
	
	/**
	 * Stop all the threads.
	 */
	protected void myStop() {	
		if (MRecv != null) MRecv.die();

		super.myStop();

		if (MRecv != null)
		{
	        synchronized (deathGuard) 
	        {
	            while( !MRecv.isDead() ) 
	            {
	                try {
	                    deathGuard.wait();
	                }
	                catch (InterruptedException ie) 
	                {
	                }
	            }
	        }
		}
	}


	/**
	 * Create local INETv4OnePort-type multicast address.
	 * @param	mcastAddressString	the string representing the multicast address.
	 */
	public void CreateMulticastAddress(String mcastAddressString) {
		multicastAddress = new INETv4OnePort(mcastAddressString);
		if (multicastAddress.getInetAddress() == null)
		{
			throw new HyperCastFatalRuntimeException ("Could not create an INETOnePort address for local multicast address.");
		}
	}

	
	/** 
	 * Sends a message to the Multicast address. <p>
	 * Algorithm: <br>
	 *  1. Create a byte array containing the information <br>
	 *  2. Create a DatagramPacket <br>
	 *  3. Send Packet. <br>
	 * 
	 * @param	msg		a protocol specific message
	 */

	public	void sendMulticastMessage(I_Message msg) {
        if (state == STOPPED || state == STARTING)
            throw new IllegalStateException("UDP_MulticastAdapter: tried to send message when adapter was not running.");

		if (MSock == null)
		{
			System.out.println("UDP_MulticastAdapter: UDP multicast socket is not activated in this adapter, message is dropped.");
		}
		else
		{
			byte[] content = msg.toByteArray(); 
	
			
			DatagramPacket Beacon = 
				new DatagramPacket(content, 
					content.length,
	                multicastAddress.getInetAddress(),
					multicastAddress.getPort());
	
			try {
				MSock.send(Beacon);
				MSentPackets++;
				MSentBytes += content.length;
			} catch (IOException e) {
				property.err.println("UDP_MulticastAdapter: Exception when sending a multicast message - " + e.toString() + "\n");
			}
		}
	}


	/**
	 * Gets the Multicast address used in this adapter.
	 */
	public I_PhysicalAddress getMulticastAddress() {
		INETv4OnePort myMulticast = new INETv4OnePort((INETv4OnePort) multicastAddress);
        return myMulticast;
    }
	
	
	/**
	 * Gets the number of packets received from the Multicast address
	 * @return	the number of packets received
	 */
	public long getMReceivedPackets() {
		return MReceivedPackets;		
	}

	
	/**
	 * Gets the number of bytes received from the Multicast address
	 */	
	public long getMReceivedBytes() {
		return MReceivedBytes;		
	}

	
	/**
	 * Gets the number of packets sent to the Multicast address
	 */	
	
	public long getMSentPackets() {
		return MSentPackets;		
	}

	
	/**
	 * Gets the number of bytes sent to the Multicast address
	 */	
	
	public long getMSentBytes() {
		return MSentBytes;		
	}
	

	/**
	 * Initialize the data structure for handling the statistics defined in this object.
	 * This class shares the StatsProcessor instance with its super class UDP_UnicastAdapter.
	 * In this method, the statistics, with each represented by a StatsElement instance, 
	 * supported in this object are added into the StatsProcessor instance.
	 */
	private void InitStatisticsStructure()
	{
		statsPro.addStatsElement("MPhysicalAddress"/* name */, new MulticastAddressQuery()/* I_Stats object */, 1/* minoccurs */, 1/* maxoccurs */);

		statsPro.addStatsElement("MPacketsReceived", new MulticastPacketsReceived(), 1, 1);

		statsPro.addStatsElement("MBytesReceived", new MulticastBytesReceived(), 1, 1);

		statsPro.addStatsElement("MPacketsSent", new MulticastPacketsSent(), 1, 1);

		statsPro.addStatsElement("MBytesSent", new MulticastBytesSent(), 1, 1);
	}

	/**
	 * This class manages simple statistic "MPhysicalAddress".
	 */
	class MulticastAddressQuery extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+multicastAddress.toString());
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "IPV4AndPort"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}


	/**
	 * This class manages simple statistic "MPacketsReceived".
	 */
	class MulticastPacketsReceived extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MReceivedPackets);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "MBytesReceived".
	 */
	class MulticastBytesReceived extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MReceivedBytes);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "MPacketsSent".
	 */
	class MulticastPacketsSent extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MSentPackets);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "MBytesSent".
	 */
	class MulticastBytesSent extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return new String(""+MSentBytes);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchema(Document, XPath)
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	
}	


/**
 * This class starts a receiver thread to receive messages from the multicast socket. 
 * Then puts the message to a MessageBuffer shared with unicast receiver thread.
 */

class MulticastReceiver implements Runnable {
    /** The buffer for received messages. */
	MessageBuffer adapterbuffer;

    /** The multicast socket. */
	MulticastSocket msocket;

    /** The adapter. */
	UDP_MulticastAdapter Parent;

    /** The offset used in calls to I_AdapterCallback.restoreMessage(). */
	int[] offset;
	
    /** The receiving thread. */
	private Thread myThread;
    /** True if the function die() has been called. */
	private boolean dying;
    /** True if <code>myThread</code> has returned from run(). */
    private boolean died;

    /** True while <code>myThread</code> is waiting to receive. */
	private boolean waiting;
    /** Guard used to protect <code>waiting</code> when it is changed. */
	private Object waitingGuard;	
	
	
	public MulticastReceiver(MessageBuffer bdbuffer, MulticastSocket msock, UDP_MulticastAdapter madapter) {
		adapterbuffer = bdbuffer;
		msocket  = msock;
		Parent = madapter;
        offset = new int[1];
		
		dying = false;	
		died = false;	

		waiting = false;
        waitingGuard = new Object();
	}

	
	/** 
	 * Start running of the Multicast thread. 
	 * Reads messages from Multicast socket and save them to the BoundedBuffer.
	 */
	public void run() {
		myThread = Thread.currentThread();
		
		DatagramPacket dpacket;
		byte[] dbuffer;
		I_Message msg;
		
				
		// Allocate memory etc
		dbuffer = new byte[Parent.MaximumPacketLength]; 
		dpacket = new DatagramPacket(dbuffer, dbuffer.length);


		try {
			msocket.setSoTimeout(0);
		} catch (SocketException e) {
		}
		
		while (!dying) {  

			// Reset the DatagramPacket size!!! Otherwise, it'll reuse last msg length
			dpacket.setLength(Parent.MaximumPacketLength);			
			
			offset[0] = 0;
			
			try 
            {
                try 
                {
                    try 
                    {
                        synchronized (waitingGuard) 
                        {
                            waiting = true;
                        }
				
                        // May block here!
                        msocket.receive(dpacket);
					
                    }
                    finally 
                    {
                        synchronized (waitingGuard) 
                        { 
                            waiting = false;
                        }
                    }
                }
                finally 
                {
                    //if (Thread.interrupted()) 
                     //   Parent.property.seriousErr.println("SERIOUS ERROR: This println should have been interrupted before it could take place. (sadly this is common...)");
                }
                    
				
				Parent.MReceivedPackets++;
				Parent.MReceivedBytes += dpacket.getLength();
				
				// put it into the MessageBuffer
                try 
                {
                    if (!dying) 
                    {
                        msg = Parent.Node.restoreMessage(dpacket.getData(), offset, dpacket.getLength());
				
                        //remove the this line, the receiveMessage(msg) will take care of discarding
                        if (msg != null) 
                        {
                            adapterbuffer.nonblockingWrite(msg);   //write to MessageBuffer
                        }
                        else 
                        {
                            Parent.property.err.println("UDP_MulticastAdapter: restoreMessage returned null! Perhaps the setting of maximum packet length is too small in your configuration.");
                            byte[] tmp_buf = new byte[dpacket.getLength()];
                            System.arraycopy(dpacket.getData(), 0, tmp_buf, 0, dpacket.getLength());
                            Parent.property.err.println("Message (hex): " + HTTP_ServerUtility.toHexString(tmp_buf));
                            Parent.property.err.println("Message src: " + dpacket.getAddress() + ":" + dpacket.getPort());
                        }
                    }
                }
                catch(Exception e) 
                {
                    Parent.property.err.println("UDP_MulticastAdapter: restoreMessage threw an exception:" + e.getMessage());
                    if (e instanceof IllegalArgumentException)
                    {
                        byte[] tmp_buf = new byte[dpacket.getLength()];
                        System.arraycopy(dpacket.getData(), 0, tmp_buf, 0, dpacket.getLength());
                        Parent.property.err.println("Message (hex): " + HTTP_ServerUtility.toHexString(tmp_buf));
                        Parent.property.err.println("Message src: " + dpacket.getAddress() + ":" + dpacket.getPort());
                    }
                    else 
                    {
                        e.printStackTrace(Parent.property.err);
                    }
                }
				
			} catch (InterruptedIOException e) {
                // thread will kill itself at top of loop.
			} catch (IOException e) {
				Parent.property.err.println("Multicast Receiver encountered IOException: " + e.toString());
			}

		}  

        synchronized (Parent.deathGuard) 
        {
            died = true;
            Parent.deathGuard.notify();
        }
	} 
	

	/**
	 * Causes <code>myThread</code> to halt eventually. 
	 */	
	
	public void die() {
         dying = true;
         synchronized (waitingGuard) {
             if (waiting)
                 myThread.interrupt();  // incase it is blocking on the wait call.
         }
     }

    /** Returns true if <code>myThread</code> will never call an I_AdapterCallback method again. */
	public boolean isDead() 
    {
        return (died || waiting);
    }
}









